==> infra-5.23.1-sources.jar: com/liveperson/infra/utils/PushMessageParser.java <==
package com.liveperson.infra.utils;

import android.content.Context;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import android.text.TextUtils;

import com.liveperson.infra.CampaignInfo;
import com.liveperson.infra.Infra;
import com.liveperson.infra.errors.ErrorCode;
import com.liveperson.infra.eventmanager.Event;
import com.liveperson.infra.log.LPLog;
import com.liveperson.infra.model.PushMessage;
import com.liveperson.infra.preferences.PushMessagePreferences;
import com.liveperson.infra.utils.markdown.HyperLinkUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Map;

import static com.liveperson.infra.errors.ErrorCode.ERR_0000010A;
import static com.liveperson.infra.errors.ErrorCode.ERR_0000010B;

/**
 * Created by ofira on 6/26/16.
 */
public class PushMessageParser {

    private static final String TAG = PushMessageParser.class.getSimpleName();

    private static final String KEY_PAYLOAD = "payload";
    private static final String KEY_MESSAGE = "message";
    private static final String KEY_TITLE = "title";
    private static final String KEY_BACKEND_SERVICE = "backendService";
    private static final String KEY_COLLAPSE = "collapse_key";
    private static final String KEY_CONVERSATION_ID = "conversationId";
    private static final String KEY_LE_ENGAGEMENT_ID = "leEngagementId";
    private static final String KEY_LE_CAMPAIGN_ID = "leCampaignId";
    private static final String KEY_IS_OUTBOUND_PUSH = "isProactivePush"; // Later ask proactive to change it to isOutboundPush
    private static final String KEY_LOOK_BACK_PERIOD = "lookBackPeriod";
    private static final String KEY_AGENT_PID = "agentPid";
    private static final String KEY_TRANSACTION_ID = "transactionId";
    private static final String KEY_BADGE_UNREAD_COUNTER = "badge";
    private static final String KEY_C2M_BACKEND = "c2m";
    private static final String KEY_EVENT = "event";
    private static final String KEY_EXPIRATION_EPOCH_TIME = "expirationEpochTime";
    private static final String KEY_DATE_CREATED = "dateCreated";

    private static final String DELIMITER = ":";

    public PushMessageParser() {}

    /**
     *
     * @param brandId
     * @param remoteMessage
     * @return push message object - can be nullable - if its not our push message or we had error parsing it.
     */
    @Nullable
    public static PushMessage parse(String brandId, Context context, Map<String, String> remoteMessage) {
        try {

            // convert data to bundle
            Bundle pushMessageBundle = convertDataToBundle(remoteMessage);

            return parsePushMessage(brandId, context, pushMessageBundle);

        } catch (Exception e) {
            LPLog.INSTANCE.e(TAG, ERR_0000010A, "Push message parsing error", e);
            return null;
        }

    }

    @NonNull
    private static PushMessage parsePushMessage(String brandId, Context context, Bundle pushMessageBundle) throws JSONException {
        String collapseKey = pushMessageBundle.getString(KEY_COLLAPSE);
        String originMessage = pushMessageBundle.getString(KEY_MESSAGE);
        // Notification from UMS will have message that contains sender and message - "sender: message". Ex "message=account_auth_code: message_from_agent"
        String splitMessage = "";
        String title = pushMessageBundle.getString(KEY_TITLE);
        String from = "";
        if (!TextUtils.isEmpty(originMessage)) {
            int splitPoint = originMessage.indexOf(DELIMITER);
            if (splitPoint != -1) {
                from = originMessage.substring(0, splitPoint).trim();
                splitMessage = originMessage.substring(splitPoint + DELIMITER.length()).trim();
                if (LinkUtils.containsMarkdownHyperlink(splitMessage)) {
                    splitMessage = HyperLinkUtils.INSTANCE.buildMessageString(splitMessage);
                }
            } else {
                splitMessage = originMessage;
            }
        }
        PushMessage pushMessage = new PushMessage(brandId, from, splitMessage);
        if (!TextUtils.isEmpty(title)) {
            pushMessage.setTitle(title);
        }
        pushMessage.setCollapseKey(collapseKey);

        String payLoadString = pushMessageBundle.getString(KEY_PAYLOAD);
        String conversationId = "";
        String backendService = "";
        String engagementId = "";
        String campaignId = "";
        boolean isOutboundPush = false;
        String lookBackPeriod = "";
        String agentPid = "";
        String unreadCounter;
        String transactionId = "";
        String messageEvent;
        long expirationEpochTime = 0;

        if(!TextUtils.isEmpty(payLoadString)) {
            JSONObject payLoadJson = new JSONObject(payLoadString);
            conversationId = payLoadJson.optString(KEY_CONVERSATION_ID);
            backendService = payLoadJson.optString(KEY_BACKEND_SERVICE);
            engagementId = payLoadJson.optString(KEY_LE_ENGAGEMENT_ID);
            campaignId = payLoadJson.optString(KEY_LE_CAMPAIGN_ID);
            isOutboundPush = payLoadJson.optBoolean(KEY_IS_OUTBOUND_PUSH);
            if (!isOutboundPush && backendService.equals(KEY_C2M_BACKEND)) {
                isOutboundPush = true;
            }
            lookBackPeriod = payLoadJson.optString(KEY_LOOK_BACK_PERIOD);
            agentPid = payLoadJson.optString(KEY_AGENT_PID);
            transactionId = payLoadJson.optString(KEY_TRANSACTION_ID);
            messageEvent = payLoadJson.optString(KEY_EVENT);
            expirationEpochTime = payLoadJson.optLong(KEY_EXPIRATION_EPOCH_TIME);

            unreadCounter = payLoadJson.optString(KEY_BADGE_UNREAD_COUNTER);
            if (!TextUtils.isEmpty(unreadCounter) && TextUtils.isDigitsOnly(unreadCounter)){
                //set the value only if we got it on the push message. otherwise we won't use it.
                pushMessage.setCurrentUnreadMessagesCounter(Integer.parseInt(unreadCounter));
            }
            if (!TextUtils.isEmpty(messageEvent)) {
                pushMessage.setMessageEvent(messageEvent);
            }
        }

        pushMessage.setConversationId(conversationId);
        pushMessage.setBackendService(backendService);
        pushMessage.setTransactionId(transactionId);
        pushMessage.setExpirationEpochTime(expirationEpochTime);

        // use transactionId as pushMessage's id if it exists
        if (!TextUtils.isEmpty(transactionId)) {
            pushMessage.setPushMessageId(transactionId);
        }

        // Check If we got campaign info from push (Sent for outbound to InApp feature)
        if (!TextUtils.isEmpty(engagementId) && !TextUtils.isEmpty(campaignId)) {
            pushMessage.setMessage(originMessage);
            String nonTagMessage = originMessage;
            if (LinkUtils.containsMarkdownHyperlink(originMessage)) {
                nonTagMessage = HyperLinkUtils.INSTANCE.buildMessageString(originMessage);
                // there is no from in Proactive message
                pushMessage.setFrom("");
            }
            pushMessage.setCampaignInfo(new CampaignInfo(Long.valueOf(campaignId), Long.valueOf(engagementId), isOutboundPush));
            if (!TextUtils.isEmpty(lookBackPeriod)) {
                pushMessage.setLookBackPeriod(Long.valueOf(lookBackPeriod));
            }
            pushMessage.setAgentPid(agentPid);
            if (context != null) {
                PushMessagePreferences.INSTANCE.init(context.getApplicationContext());
                PushMessagePreferences.INSTANCE.setCachedPushMessage(brandId, pushMessage);
                Infra.instance.getEventManagerService().logEvent(brandId, transactionId, true, Event.DELIVERY, null, null, context.getApplicationContext());
            }
            // return removed #md tag message to the host app
            pushMessage.setMessage(nonTagMessage);
        }
        return pushMessage;
    }

    /**
     * Parse response to get pending proactive messages
     * @param brandId
     * @param context
     * @param jsonString
     * @return
     */
    public static @NonNull ArrayList<PushMessage> parsePendingProactiveMessages(String brandId, Context context, String jsonString) {
        ArrayList<PushMessage> pushMessageList = new ArrayList<>();
        try {
            JSONObject root = new JSONObject(jsonString);
            JSONArray messages = root.getJSONArray("messages");
            if (messages == null) {
                return pushMessageList;
            }

            // Clear all cached push messages before saving newly fetched messages
            PushMessagePreferences.INSTANCE.init(context);
            PushMessagePreferences.INSTANCE.clearAll();

            PushMessage pushMessage;
            for (int i = 0; i < messages.length(); i++) {
                pushMessage = parseProactiveData(brandId, context, messages.getJSONObject(i));
                if (pushMessage != null) {
                    pushMessageList.add(pushMessage);
                }
            }
        } catch (JSONException ex) {
            LPLog.INSTANCE.e(TAG, ErrorCode.ERR_0000016I, "Failed to parse proactive messages", ex);
            return pushMessageList;
        }

        return pushMessageList;
    }

    /**
     * Parse each item to get a proactive message
     * @param brandId
     * @param applicationContext
     * @param messageJson
     * @return
     */
    private static PushMessage parseProactiveData(String brandId, Context applicationContext, JSONObject messageJson) {
        if (messageJson == null) {
            return null;
        }

        PushMessage pushMessage;

        try {
            JSONObject data = messageJson.getJSONObject("data");
            String payloadStr = data.getString("payload");
            JSONObject payload = new JSONObject(payloadStr);
            String message = data.getString("message");

            String conversationId;
            String backendService;
            String engagementId;
            String campaignId;
            boolean isOutboundPush;
            String lookBackPeriod;
            String agentPid;
            String transactionId;
            String messageEvent;
            long expirationEpochTime;
            long dateCreated;

            conversationId = payload.optString(KEY_CONVERSATION_ID);
            backendService = payload.optString(KEY_BACKEND_SERVICE);
            engagementId = payload.optString(KEY_LE_ENGAGEMENT_ID);
            campaignId = payload.optString(KEY_LE_CAMPAIGN_ID);
            isOutboundPush = payload.optBoolean(KEY_IS_OUTBOUND_PUSH);
            if (!isOutboundPush && backendService.equals(KEY_C2M_BACKEND)) {
                isOutboundPush = true;
            }
            lookBackPeriod = payload.optString(KEY_LOOK_BACK_PERIOD);
            agentPid = payload.optString(KEY_AGENT_PID);
            transactionId = payload.optString(KEY_TRANSACTION_ID);
            messageEvent = payload.optString(KEY_EVENT);
            expirationEpochTime = payload.optLong(KEY_EXPIRATION_EPOCH_TIME);
            dateCreated = payload.optLong(KEY_DATE_CREATED);

            pushMessage = new PushMessage(brandId, "", message, dateCreated);
            if (!TextUtils.isEmpty(messageEvent)) {
                pushMessage.setMessageEvent(messageEvent);
            }
            pushMessage.setConversationId(conversationId);
            pushMessage.setBackendService(backendService);
            pushMessage.setTransactionId(transactionId);
            pushMessage.setPushMessageId(transactionId);
            pushMessage.setAgentPid(agentPid);
            pushMessage.setExpirationEpochTime(expirationEpochTime);

            if (!TextUtils.isEmpty(engagementId) && !TextUtils.isEmpty(campaignId)) {
                pushMessage.setCampaignInfo(new CampaignInfo(Long.valueOf(campaignId), Long.valueOf(engagementId), isOutboundPush));
                String nonTagMessage = message;
                if (LinkUtils.containsMarkdownHyperlink(message)) {
                    nonTagMessage = HyperLinkUtils.INSTANCE.buildMessageString(message);
                }
                if (!TextUtils.isEmpty(lookBackPeriod)) {
                    pushMessage.setLookBackPeriod(Long.valueOf(lookBackPeriod));
                }
                if (applicationContext != null) {
                    PushMessagePreferences.INSTANCE.init(applicationContext);
                    PushMessagePreferences.INSTANCE.setCachedPushMessage(brandId, pushMessage);
                    Infra.instance.getEventManagerService().logEvent(brandId, transactionId, true, Event.DELIVERY, null, null, applicationContext);
                }
                // return removed #md tag message to the host app
                pushMessage.setMessage(nonTagMessage);
            }
        } catch (JSONException ex) {
            LPLog.INSTANCE.e(TAG, ErrorCode.ERR_0000016I, "Failed to parse proactive data", ex);
            return null;
        }

        return pushMessage;
    }

    /**
     * Convert the Map received from FCM service to a Bundle (required by the PushMessageParser)
     * @param data
     * @return
     */
    public static Bundle convertDataToBundle(Map<String, String> data) {

        Bundle bundle = new Bundle();

        for (Map.Entry<String, String> entry : data.entrySet()) {

            bundle.putString(entry.getKey(), entry.getValue());
        }

        return bundle;
    }

    public static PushMessage parseBundle(String brandId, Context context, Bundle data) {
        try {

            return parsePushMessage(brandId, context, data);

        } catch (Exception e) {
            LPLog.INSTANCE.e(TAG, ERR_0000010B, "Push message parsing error", e);
            return null;
        }
    }
}

==> infra-5.25.0-sources.jar: com/liveperson/infra/utils/PushMessageParser.java <==
package com.liveperson.infra.utils;

import android.content.Context;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.VisibleForTesting;

import android.text.TextUtils;

import com.liveperson.infra.CampaignInfo;
import com.liveperson.infra.Infra;
import com.liveperson.infra.errors.ErrorCode;
import com.liveperson.infra.eventmanager.Event;
import com.liveperson.infra.html.HtmlUtils;
import com.liveperson.infra.log.LPLog;
import com.liveperson.infra.md.MarkdownUtils;
import com.liveperson.infra.model.PushMessage;
import com.liveperson.infra.model.types.NotificationType;
import com.liveperson.infra.preferences.PushMessagePreferences;
import com.liveperson.infra.utils.markdown.HyperLinkUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Map;

import static com.liveperson.infra.errors.ErrorCode.ERR_0000010A;
import static com.liveperson.infra.errors.ErrorCode.ERR_0000010B;

/**
 * Created by ofira on 6/26/16.
 */
public class PushMessageParser {

    private static final String TAG = PushMessageParser.class.getSimpleName();

    private static final String KEY_PAYLOAD = "payload";
    private static final String KEY_MESSAGE = "message";
    private static final String KEY_TITLE = "title";
    private static final String KEY_BACKEND_SERVICE = "backendService";
    private static final String KEY_COLLAPSE = "collapse_key";
    private static final String KEY_CONVERSATION_ID = "conversationId";
    private static final String KEY_LE_ENGAGEMENT_ID = "leEngagementId";
    private static final String KEY_LE_CAMPAIGN_ID = "leCampaignId";
    private static final String KEY_IS_OUTBOUND_PUSH = "isProactivePush"; // Later ask proactive to change it to isOutboundPush
    private static final String KEY_LOOK_BACK_PERIOD = "lookBackPeriod";
    private static final String KEY_AGENT_PID = "agentPid";
    private static final String KEY_TRANSACTION_ID = "transactionId";
    private static final String KEY_BADGE_UNREAD_COUNTER = "badge";
    private static final String KEY_C2M_BACKEND = "c2m";
    private static final String KEY_EVENT = "event";
    private static final String KEY_EXPIRATION_EPOCH_TIME = "expirationEpochTime";
    private static final String KEY_DATE_CREATED = "dateCreated";
    private static final String VAL_PRMSG_BACKEND = "prmsg";

    private static final String DELIMITER = ":";

    public PushMessageParser() {}

    /**
     *
     * @param brandId
     * @param remoteMessage
     * @return push message object - can be nullable - if its not our push message or we had error parsing it.
     */
    @Nullable
    public static PushMessage parse(String brandId, Context context, Map<String, String> remoteMessage) {
        try {

            // convert data to bundle
            Bundle pushMessageBundle = convertDataToBundle(remoteMessage);

            return parsePushMessage(brandId, context, pushMessageBundle);

        } catch (Exception e) {
            LPLog.INSTANCE.e(TAG, ERR_0000010A, "Push message parsing error", e);
            return null;
        }

    }

    @NonNull
    @VisibleForTesting
    static PushMessage parsePushMessage(String brandId, Context context, Bundle pushMessageBundle) throws JSONException {
        String collapseKey = pushMessageBundle.getString(KEY_COLLAPSE);
        String originMessage = pushMessageBundle.getString(KEY_MESSAGE);
        // Notification from UMS will have message that contains sender and message - "sender: message". Ex "message=account_auth_code: message_from_agent"
        String splitMessage = "";
        String title = pushMessageBundle.getString(KEY_TITLE);
        String from = "";
        if (!TextUtils.isEmpty(originMessage)) {
            int splitPoint = originMessage.indexOf(DELIMITER);
            if (splitPoint != -1) {
                from = originMessage.substring(0, splitPoint).trim();
                splitMessage = originMessage.substring(splitPoint + DELIMITER.length()).trim();
                if (LinkUtils.containsMarkdownHyperlink(splitMessage)) {
                    splitMessage = HyperLinkUtils.INSTANCE.buildMessageString(splitMessage);
                }
            } else {
                splitMessage = originMessage;
            }
        }
        PushMessage pushMessage = new PushMessage(brandId, from, splitMessage);
        if (!TextUtils.isEmpty(title)) {
            pushMessage.setTitle(title);
        }
        pushMessage.setCollapseKey(collapseKey);

        String payLoadString = pushMessageBundle.getString(KEY_PAYLOAD);
        String conversationId = "";
        String backendService = "";
        String engagementId = "";
        String campaignId = "";
        boolean isOutboundPush = false;
        String lookBackPeriod = "";
        String agentPid = "";
        String unreadCounter;
        String transactionId = "";
        String messageEvent;
        long expirationEpochTime = 0;

        if(!TextUtils.isEmpty(payLoadString)) {
            JSONObject payLoadJson = new JSONObject(payLoadString);
            conversationId = payLoadJson.optString(KEY_CONVERSATION_ID);
            backendService = payLoadJson.optString(KEY_BACKEND_SERVICE);
            engagementId = payLoadJson.optString(KEY_LE_ENGAGEMENT_ID);
            campaignId = payLoadJson.optString(KEY_LE_CAMPAIGN_ID);
            isOutboundPush = payLoadJson.optBoolean(KEY_IS_OUTBOUND_PUSH);
            if (!isOutboundPush && backendService.equals(KEY_C2M_BACKEND)) {
                isOutboundPush = true;
            }
            lookBackPeriod = payLoadJson.optString(KEY_LOOK_BACK_PERIOD);
            agentPid = payLoadJson.optString(KEY_AGENT_PID);
            transactionId = payLoadJson.optString(KEY_TRANSACTION_ID);
            messageEvent = payLoadJson.optString(KEY_EVENT);
            expirationEpochTime = payLoadJson.optLong(KEY_EXPIRATION_EPOCH_TIME);

            unreadCounter = payLoadJson.optString(KEY_BADGE_UNREAD_COUNTER);
            if (!TextUtils.isEmpty(unreadCounter) && TextUtils.isDigitsOnly(unreadCounter)){
                //set the value only if we got it on the push message. otherwise we won't use it.
                pushMessage.setCurrentUnreadMessagesCounter(Integer.parseInt(unreadCounter));
            }
            if (!TextUtils.isEmpty(messageEvent)) {
                pushMessage.setMessageEvent(messageEvent);
            }
        }

        pushMessage.setConversationId(conversationId);
        pushMessage.setBackendService(backendService);
        pushMessage.setTransactionId(transactionId);
        pushMessage.setExpirationEpochTime(expirationEpochTime);

        // use transactionId as pushMessage's id if it exists
        if (!TextUtils.isEmpty(transactionId)) {
            pushMessage.setPushMessageId(transactionId);
        }

        // Check If we got campaign info from push (Sent for outbound to InApp feature)
        if (!TextUtils.isEmpty(engagementId) && !TextUtils.isEmpty(campaignId)) {
            // Set notification type based on backend service, default will be REGULAR
            if (isOutboundPush) {
                pushMessage.setNotificationType(NotificationType.C2M);
                if (VAL_PRMSG_BACKEND.equalsIgnoreCase(backendService)) {
                    pushMessage.setNotificationType(NotificationType.PROACTIVE);
                }
            }
            pushMessage.setMessage(originMessage);
            String nonTagMessage = parseHtmlContent(originMessage);
            pushMessage.setFrom("");
            pushMessage.setCampaignInfo(new CampaignInfo(Long.valueOf(campaignId), Long.valueOf(engagementId), isOutboundPush));
            if (!TextUtils.isEmpty(lookBackPeriod)) {
                pushMessage.setLookBackPeriod(Long.valueOf(lookBackPeriod));
            }
            pushMessage.setAgentPid(agentPid);
            if (context != null) {
                PushMessagePreferences.INSTANCE.init(context.getApplicationContext());
                PushMessagePreferences.INSTANCE.setCachedPushMessage(brandId, pushMessage);
                Infra.instance.getEventManagerService().logEvent(brandId, transactionId, true, Event.DELIVERY, null, null, context.getApplicationContext());
            }
            // return removed #md tag message to the host app
            pushMessage.setMessage(nonTagMessage);
        }
        return pushMessage;
    }

    @VisibleForTesting
    static String parseHtmlContent(String message) {
        String htmlMessage;
        try {
            htmlMessage = MarkdownUtils.markdownToHtml(message);
        } catch (Throwable exception) {
            LPLog.INSTANCE.d(TAG, "Failed to parse markdown", exception);
            htmlMessage = message;
        }
        return HtmlUtils.convertHtmlToStyledString(htmlMessage).toString();
    }

    /**
     * Parse response to get pending proactive messages
     * @param brandId
     * @param context
     * @param jsonString
     * @return
     */
    public static @NonNull ArrayList<PushMessage> parsePendingProactiveMessages(String brandId, Context context, String jsonString) {
        ArrayList<PushMessage> pushMessageList = new ArrayList<>();
        try {
            JSONObject root = new JSONObject(jsonString);
            JSONArray messages = root.getJSONArray("messages");
            if (messages == null) {
                return pushMessageList;
            }

            // Clear all cached push messages before saving newly fetched messages
            PushMessagePreferences.INSTANCE.init(context);
            PushMessagePreferences.INSTANCE.clearAll();

            PushMessage pushMessage;
            for (int i = 0; i < messages.length(); i++) {
                pushMessage = parseProactiveData(brandId, context, messages.getJSONObject(i));
                if (pushMessage != null) {
                    pushMessageList.add(pushMessage);
                }
            }
        } catch (JSONException ex) {
            LPLog.INSTANCE.e(TAG, ErrorCode.ERR_0000016I, "Failed to parse proactive messages", ex);
            return pushMessageList;
        }

        return pushMessageList;
    }

    /**
     * Parse each item to get a proactive message
     * @param brandId
     * @param applicationContext
     * @param messageJson
     * @return
     */
    @VisibleForTesting
    static PushMessage parseProactiveData(String brandId, Context applicationContext, JSONObject messageJson) {
        if (messageJson == null) {
            return null;
        }

        PushMessage pushMessage;

        try {
            JSONObject data = messageJson.getJSONObject("data");
            String payloadStr = data.getString("payload");
            JSONObject payload = new JSONObject(payloadStr);
            String message = data.getString("message");

            String conversationId;
            String backendService;
            String engagementId;
            String campaignId;
            boolean isOutboundPush;
            String lookBackPeriod;
            String agentPid;
            String transactionId;
            String messageEvent;
            long expirationEpochTime;
            long dateCreated;

            conversationId = payload.optString(KEY_CONVERSATION_ID);
            backendService = payload.optString(KEY_BACKEND_SERVICE);
            engagementId = payload.optString(KEY_LE_ENGAGEMENT_ID);
            campaignId = payload.optString(KEY_LE_CAMPAIGN_ID);
            isOutboundPush = payload.optBoolean(KEY_IS_OUTBOUND_PUSH);
            if (!isOutboundPush && backendService.equals(KEY_C2M_BACKEND)) {
                isOutboundPush = true;
            }
            lookBackPeriod = payload.optString(KEY_LOOK_BACK_PERIOD);
            agentPid = payload.optString(KEY_AGENT_PID);
            transactionId = payload.optString(KEY_TRANSACTION_ID);
            messageEvent = payload.optString(KEY_EVENT);
            expirationEpochTime = payload.optLong(KEY_EXPIRATION_EPOCH_TIME);
            dateCreated = payload.optLong(KEY_DATE_CREATED);

            pushMessage = new PushMessage(brandId, "", message, dateCreated);
            if (!TextUtils.isEmpty(messageEvent)) {
                pushMessage.setMessageEvent(messageEvent);
            }
            pushMessage.setConversationId(conversationId);
            pushMessage.setBackendService(backendService);
            pushMessage.setTransactionId(transactionId);
            pushMessage.setPushMessageId(transactionId);
            pushMessage.setAgentPid(agentPid);
            pushMessage.setExpirationEpochTime(expirationEpochTime);

            if (!TextUtils.isEmpty(engagementId) && !TextUtils.isEmpty(campaignId)) {
                // Set notification type based on backend service, default will be REGULAR
                if (isOutboundPush) {
                    pushMessage.setNotificationType(NotificationType.C2M);
                    if (VAL_PRMSG_BACKEND.equalsIgnoreCase(backendService)) {
                        pushMessage.setNotificationType(NotificationType.PROACTIVE);
                    }
                }
                pushMessage.setCampaignInfo(new CampaignInfo(Long.valueOf(campaignId), Long.valueOf(engagementId), isOutboundPush));
                String nonTagMessage = message;
                if (LinkUtils.containsMarkdownHyperlink(message)) {
                    nonTagMessage = HyperLinkUtils.INSTANCE.buildMessageString(message);
                }
                if (!TextUtils.isEmpty(lookBackPeriod)) {
                    pushMessage.setLookBackPeriod(Long.valueOf(lookBackPeriod));
                }
                if (applicationContext != null) {
                    PushMessagePreferences.INSTANCE.init(applicationContext);
                    PushMessagePreferences.INSTANCE.setCachedPushMessage(brandId, pushMessage);
                    Infra.instance.getEventManagerService().logEvent(brandId, transactionId, true, Event.DELIVERY, null, null, applicationContext);
                }
                // return removed #md tag message to the host app
                pushMessage.setMessage(nonTagMessage);
            }
        } catch (JSONException ex) {
            LPLog.INSTANCE.e(TAG, ErrorCode.ERR_0000016I, "Failed to parse proactive data", ex);
            return null;
        }

        return pushMessage;
    }

    /**
     * Convert the Map received from FCM service to a Bundle (required by the PushMessageParser)
     * @param data
     * @return
     */
    public static Bundle convertDataToBundle(Map<String, String> data) {

        Bundle bundle = new Bundle();

        for (Map.Entry<String, String> entry : data.entrySet()) {

            bundle.putString(entry.getKey(), entry.getValue());
        }

        return bundle;
    }

    public static PushMessage parseBundle(String brandId, Context context, Bundle data) {
        try {

            return parsePushMessage(brandId, context, data);

        } catch (Exception e) {
            LPLog.INSTANCE.e(TAG, ERR_0000010B, "Push message parsing error", e);
            return null;
        }
    }
}
